@@ -1,16 +1,16 @@
-#' Fitting Multi-Staged Bayesian Regression Model with Unmeasured Confounders
+#' Fitting Multilevel Bayesian Regression Model with Unmeasured Confounders
 #'
-#' [unm_glm()] fits a multi-staged Bayesian regression model that accounts for
+#' [unm_glm()] fits a multilevel Bayesian regression model that accounts for
 #' unmeasured confounders. Users can input model information into [unm_glm()] in
 #' a similar manner as they would for the standard [stats::glm()] function,
 #' providing arguments like `formula`, `family`, and `data`. Results are stored
 #' as MCMC iterations.
 #'
-#' @param form1 The formula specification for the response model (stage I)
+#' @param form1 The formula specification for the response model (Level I)
 #' @param form2 The formula specification for the first unmeasured confounder
-#'   model (stage II)
+#'   model (Level II)
 #' @param form3 The formula specification for the second unmeasured confounder
-#'   model (stage III)
+#'   model (Level III)
 #' @param family1,family2,family3 The family object, communicating the types of
 #'   models to be used for response (`form1`) and unmeasured confounder (`form2,
 #'   form3`) models. See [stats::family()] for details
@@ -29,12 +29,10 @@
 #'   `options(unm_progress.bar = "text")`.
 #' @param code_only Should only the code be created?
 #' @param priors Custom priors to use on regression coefficients, see examples.
-#' @param
-#'   response_nuisance_priors,confounder1_nuisance_priors,confounder2_nuisance_priors
+#' @param response_nuisance_priors,confounder1_nuisance_priors,confounder2_nuisance_priors
 #'   JAGS code for the nuisance priors on parameters in a JAGS model (see
 #'   examples)
-#' @param
-#'   response_params_to_track,confounder1_params_to_track,confounder2_params_to_track
+#' @param response_params_to_track,confounder1_params_to_track,confounder2_params_to_track
 #'   Additional parameters to track when nuisance parameter priors are used (see
 #'   examples)
 #' @param ... Additional arguments to pass into [rjags::jags.model()], such as
@@ -46,20 +44,20 @@
 #' @seealso [runm()], [rjags::dic.samples()]
 #' @examples
 #'
-#' \donttest{
-#'
-#' # ~~ One Unmeasured Confounder Examples (II-Stage Model) ~~
+#' # ~~ One Unmeasured Confounder Examples (II-Level Model) ~~
 #'
 #'
 #' # normal response, normal confounder model with internally validated data
 #' (df <- runm(20, response = "norm"))
 #'
 #' (unm_mod <- unm_glm(
-#'   y ~ x + z1 + z2 + z3 + u1,  family1 = gaussian(),
-#'   u1 ~ x + z1 + z2 + z3,      family2 = gaussian(),
+#'   form1 = y ~ x + z1 + z2 + z3 + u1,  family1 = gaussian(),
+#'   form2 = u1 ~ x + z1 + z2 + z3,      family2 = gaussian(),
 #'   data = df
 #' ))
 #'
+#' \dontrun{ # reduce cran check time
+#'
 #' (unm_mod <- unm_glm(
 #'   y ~ .,      family1 = gaussian(),
 #'   u1 ~ . - y, family2 = gaussian(),
@@ -79,12 +77,350 @@
 #'
 #'
 #'
+#'
+#' # a normal-normal model - external validation
+#' (df <- runm(c(10, 10), type = "ext", response = "norm"))
+#'
+#' unm_glm(
+#'   y ~ x + z1 + z2 + z3 + u1,  family1 = gaussian(),
+#'   u1 ~ x + z1 + z2 + z3,      family2 = gaussian(),
+#'   data = df
+#' )
+#'
+#'
+#'
+#' # setting custom priors
+#' unm_glm(
+#'   y ~ .,      family1 = gaussian(),
+#'   u1 ~ . - y, family2 = gaussian(),
+#'   data = df,
+#'   code_only = TRUE
+#' )
+#'
+#' unm_glm(
+#'   y ~ .,      family1 = gaussian(),
+#'   u1 ~ . - y, family2 = gaussian(),
+#'   data = df,
+#'   code_only = FALSE,
+#'   priors = c("lambda[u1]" = "dnorm(1, 10)"),
+#'   response_nuisance_priors = "tau_{y} <- sigma_{y}^-2; sigma_{y} ~ dunif(0, 100)",
+#'   response_params_to_track = "sigma_{y}",
+#'   confounder1_nuisance_priors = "tau_{u1} <- sigma_{u1}^-2; sigma_{u1} ~ dunif(0, 100)",
+#'   confounder1_params_to_track = "sigma_{u1}"
+#' )
+#'
+#'
+#' # turn progress tracking on
+#' options("unm_progress.bar" = "text")
+#'
+#'
+#'
+#' # more complex functional forms _for non-confounder predictors only_
+#' # zero-intercept model
+#' unm_glm(
+#'   y ~ . - 1,
+#'   u1 ~ . - y,
+#'   family1 = gaussian(),
+#'   family2 = gaussian(),
+#'   data = df
+#' )
+#' glm(y ~ . - 1, data = df)
+#'
+#' # polynomial model
+#' unm_glm(
+#'   y ~ x + poly(z1, 2) + u1,
+#'   u1 ~ x + z1,
+#'   family1 = gaussian(),
+#'   family2 = gaussian(),
+#'   data = df
+#' )
+#' glm(y ~ x + poly(z1, 2), data = df)
+#'
+#' # interaction model
+#' unm_glm(
+#'   y ~ x*z1 + u1, family1 = gaussian(),
+#'   u1 ~ x*z1,     family2 = gaussian(),
+#'   data = df
+#' )
+#' glm(y ~ x*z1, data = df)
+#'
+#'
+#'
+#' # a binomial-binomial model
+#' (df <- runm(
+#'   50,
+#'   missing_prop = .75,
+#'   response = "bin",
+#'   unmeasured_fam_list = list("bin"),
+#'   unmeasured_param_list = list(.5)
+#' ))
+#' (unm_mod <- unm_glm(
+#'   y ~ .,
+#'   u1 ~ . - y,
+#'   family1 = binomial(),
+#'   family2 = binomial(),
+#'   data = df
+#' ))
+#' glm(y ~ . - u1, family = binomial(), data = df)
+#'
+#'
+#'
+#' # a poisson-normal model
+#' (df <- runm(
+#'   25,
+#'   response = "pois",
+#'   response_model_coefs = c("int" = -1, "z" = .5, "u1" = .5, "x" = .5),
+#'   treatment_model_coefs = c("int" = -1, "z" = .5, "u1" = .5),
+#'   covariate_fam_list = list("norm"),
+#'   covariate_param_list = list(c(mean = 0, sd = 1)),
+#'   unmeasured_fam_list = list("norm"),
+#'   unmeasured_param_list = list(c(0, 1))
+#' ))
+#'
+#' (unm_mod <- unm_glm(
+#'   y ~ x + z + u1 + offset(log(t)),
+#'   u1 ~ x + z,
+#'   family1 = poisson(),
+#'   family2 = gaussian(),
+#'   data = df
+#' ))
+#' glm(y ~ x + z + offset(log(t)), family = poisson(), data = df)
+#'
+#'
+#'
+#' # a poisson-binomial model
+#' (df <- runm(
+#'   25,
+#'   response = "pois",
+#'   response_model_coefs = c("int" = -1, "z" = .5, "u1" = .5, "x" = .5),
+#'   treatment_model_coefs = c("int" = -1, "z" = .5, "u1" = .5),
+#'   covariate_fam_list = list("norm"),
+#'   covariate_param_list = list(c(mean = 0, sd = 1)),
+#'   unmeasured_fam_list = list("bin"),
+#'   unmeasured_param_list = list(.5)
+#' ))
+#'
+#' (unm_mod <- unm_glm(
+#'   y ~ x + z + u1 + offset(log(t)), family1 = poisson(),
+#'   u1 ~ x + z,                      family2 = binomial(),
+#'   data = df
+#' ))
+#'
+#' glm(y ~ x + z + offset(log(t)), family = poisson(), data = df)
+#'
+#'
+#'
+#' # a gamma-normal model
+#' (df <- runm(
+#'   25,
+#'   response = "gam",
+#'   response_model_coefs = c("int" = -1, "z" = .5, "u1" = .5, "x" = .5),
+#'   treatment_model_coefs = c("int" = -1, "z" = .5, "u1" = .5),
+#'   covariate_fam_list = list("norm"),
+#'   covariate_param_list = list(c(mean = 0, sd = 1)),
+#'   unmeasured_fam_list = list("norm"),
+#'   unmeasured_param_list = list(c(0, 1))
+#' ))
+#'
+#' (unm_mod <- unm_glm(
+#'   y ~ x + z + u1, family1 = Gamma(),
+#'   u1 ~ x + z,     family2 = gaussian(),
+#'   data = df
+#' ))
+#'
+#' glm(y ~ x + z, family = Gamma(link = "log"), data = df)
+#'
+#'
+#'
+#' # a gamma-binomial model
+#' (df <- runm(
+#'   25,
+#'   response = "gam",
+#'   response_model_coefs = c("int" = -1, "z" = .5, "u1" = .5, "x" = .5),
+#'   treatment_model_coefs = c("int" = -1, "z" = .5, "u1" = .5),
+#'   covariate_fam_list = list("norm"),
+#'   covariate_param_list = list(c(mean = 0, sd = 1)),
+#'   unmeasured_fam_list = list("bin"),
+#'   unmeasured_param_list = list(.5)
+#' ))
+#'
+#' (unm_mod <- unm_glm(
+#'   y ~ x + z + u1, family1 = Gamma(),
+#'   u1 ~ x + z,     family2 = binomial(),
+#'   data = df
+#' ))
+#' print(df, n = 25)
+#'
+#' glm(y ~ x + z, family = Gamma(link = "log"), data = df)
+#'
+#'
+#'
+#' # the output of unm_glm() is classed jags output
+#'
+#' (df <- runm(20, response = "norm"))
+#' (unm_mod <- unm_glm(
+#'   y ~ .,
+#'   u1 ~ . - y,
+#'   family1 = gaussian(),
+#'   family2 = gaussian(),
+#'   data = df)
+#' )
+#' class(unm_mod)
+#' jags_code(unm_mod)
+#' unm_glm(y ~ ., u1 ~ . - y, data = df, code_only = TRUE)
+#'
+#'
+#'
+#'
+#'
+#' # visualizing output
+#' library("ggplot2")
+#' library("bayesplot"); bayesplot_theme_set(ggplot2::theme_minimal())
+#' mcmc_hist(unm_mod, facet_args = list(labeller = label_parsed))
+#' mcmc_hist(unm_mod)
+#' mcmc_trace(unm_mod, facet_args = list(labeller = label_parsed))
+#'
+#' # more extensive visualization with the tidyverse
+#' mcmc_intervals(unm_mod, prob = .90) +
+#'   geom_point(
+#'     aes(value, name), data = tibble::enframe(attr(df, "params")),
+#'     color = "red", fill = "pink", size = 4, shape = 21
+#'   )
+#'
+#'
+#' library("dplyr")
+#' library("tidyr")
+#' unm_mod %>%
+#'   as.matrix() %>%
+#'   as_tibble() %>%
+#'   pivot_longer(everything(), names_to = "var", values_to = "val") %>%
+#'   ggplot(aes("0", val)) +
+#'   geom_jitter() +
+#'   geom_point(
+#'     aes("0", value), data = tibble::enframe(attr(df, "params"), name = "var"),
+#'     color = "red", fill = "pink", size = 4, shape = 21
+#'   ) +
+#'   coord_flip() +
+#'   facet_grid(var ~ ., scales = "free_y", labeller = label_parsed) +
+#'   theme_bw() +
+#'   theme(
+#'     axis.title = element_blank(),
+#'     axis.text.y = element_blank(), axis.ticks.y = element_blank(),
+#'     strip.text.y = element_text(angle = 0)
+#'   )
+#'
+#'
+#' # getting draws out
+#' (samps <- posterior::as_draws_df(unm_mod))
+#' samps$`.chain`
+#' samps$`.iteration`
+#' samps$`.draw`
+#'
+#'
+#'
+#' # implementation is variable-name independent
+#' (df <- runm(100, response = "norm"))
+#' df$ht <- df$y
+#' df$age <- df$u1
+#' df$biom <- df$x
+#' (unm_mod <- unm_glm(
+#'   ht ~ x + biom + age,
+#'   age ~ x + biom,
+#'   data = df,
+#'   family1 = gaussian(),
+#'   family2 = gaussian()
+#' ))
+#' jags_code(unm_mod)
+#'
+#' # ~~ Two Unmeasured Confounders Examples (III-Level Model) ~~
+#' # a normal-normal-normal model - internal validation
+#' (df <- runm(
+#'   50,
+#'   missing_prop = .75,
+#'   response = "norm",
+#'   response_model_coefs = c("int" = -1, "z" = .5,
+#'                            "u1" = .5, "u2" = .5, "x" = .5),
+#'   treatment_model_coefs = c("int" = -1, "z" = .5,
+#'                             "u1" = .5, "u2" = .5),
+#'   covariate_fam_list = list("norm"),
+#'   covariate_param_list = list(c(mean = 0, sd = 1)),
+#'   unmeasured_fam_list = list("norm", "norm"),
+#'   unmeasured_param_list = list(c(0, 1), c(0, 1))
+#' ))
+#' (unm_mod <- unm_glm(
+#'   y ~ x + z + u1 + u2,
+#'   u1 ~ x + z + u2,
+#'   u2 ~ x + z,
+#'   family1 = gaussian(),
+#'   family2 = gaussian(),
+#'   family3 = gaussian(),
+#'   data = df
+#' ))
+#'
+#' glm(y ~ x + z, data = df)
+#' coef(unm_mod)
+#'
+#' unm_glm(
+#'   y ~ x + z + u1 + u2, family1 = gaussian(),
+#'   u1 ~ x + z + u2,   family2 = gaussian(),
+#'   u2 ~ x + z,        family3 = gaussian(),
+#'   data = df,
+#'   code_only = TRUE
+#' )
+#'
+#'
+#'
+#' # a normal-normal-normal model - external validation
+#' (df <- runm(
+#'   c(20, 20),
+#'   type = "ext",
+#'   response = "norm",
+#'   response_model_coefs = c("int" = -1, "z" = .5,
+#'                            "u1" = .5, "u2" = .5, "x" = .5),
+#'   treatment_model_coefs = c("int" = -1, "z" = .5,
+#'                             "u1" = .5, "u2" = .5),
+#'   covariate_fam_list = list("norm"),
+#'   covariate_param_list = list(c(mean = 0, sd = 1)),
+#'   unmeasured_fam_list = list("norm", "norm"),
+#'   unmeasured_param_list = list(c(0, 1), c(0, 1))
+#' ))
+#' (unm_mod <- unm_glm(
+#'   y ~ x + z + u1 + u2,  family1 = gaussian(),
+#'   u1 ~ x + z + u2,      family2 = gaussian(),
+#'   u2 ~ x + z,           family3 = gaussian(),
+#'   data = df
+#' ))
+#'
+#'
+#'
+#' # a binomial-binomial-binomial model - internal validation
+#' (df <- runm(
+#'   25,
+#'   response = "bin",
+#'   response_model_coefs = c("int" = -1, "z" = .5,
+#'                            "u1" = .5, "u2" = .5, "x" = .5),
+#'   treatment_model_coefs = c("int" = -1, "z" = .5,
+#'                             "u1" = .5, "u2" = .5),
+#'   covariate_fam_list = list("norm"),
+#'   covariate_param_list = list(c(mean = 0, sd = 1)),
+#'   unmeasured_fam_list = list("bin", "bin"),
+#'   unmeasured_param_list = list(.5, .75)
+#' ))
+#' unm_glm(y ~ x + z + u1 + u2, family1 = binomial(),
+#'         u1 ~ x + z + u2,     family2 = binomial(),
+#'         u2 ~ x + z,          family3 = binomial(),
+#'         data = df,
+#'         code_only = TRUE
+#' )
+#'
+#'
 #' }
 #'
 
 
 #' @export
 #' @rdname unm_glm
+
 unm_glm <- function(
     form1, form2 = NULL, form3 = NULL,
     family1 = binomial(), family2 = NULL, family3 = NULL,
@@ -110,8 +446,8 @@ unm_glm <- function(
   u1 <- if (inherits(form2, "formula")) deparse(form2[[2]]) else NULL # e.g. "u1", character name of confounding var
   u2 <- if (inherits(form3, "formula")) deparse(form3[[2]]) else NULL # e.g. "u2", character name of confounding var
 
-
-  if (grepl(paste(g("\\b{u1}\\b"), g("\\b{u2}\\b"), sep = "|"), deparse(form1[[3]]))) {
+  if (!(is.null(u1) || is.null(u2)) &&
+      grepl(paste(g("\\b{u1}\\b"), g("\\b{u2}\\b"), sep = "|"), deparse(form1[[3]]))) {
     conf_piece <- "+ inprod(U[i,], lambda)"
   } else {
     conf_piece <- ""
@@ -137,7 +473,7 @@ unm_glm <- function(
 
     response_nuisance_priors <- switch(
       family1$family,
-      "gaussian" = g("tau_{y} ~ dt(0, 1, 3) I(0, )"),
+      "gaussian" = g("tau_{y} ~ dgamma(0.001, 0.001)"),
       "binomial" = " ",
       "Gamma" = g("alpha_{y} ~ dgamma(.1, .1)"),
       "poisson" = " "
@@ -158,12 +494,14 @@ unm_glm <- function(
 
   }
 
-  if (!is.null(u2)) {
+  if (!is.null(u2) && grepl(paste(g("\\b{u2}\\b")), deparse(form2[[3]]))) {
     conf2_piece <- "+ inprod(U[i, 2], zeta)"
   } else {
     conf2_piece <- ""
   }
 
+
+
   confounder1_model_code <- switch(
     family2$family,
     "gaussian" = g("U[i,1] ~ dnorm(mu_{u1}[i], tau_{u1})
@@ -177,7 +515,7 @@ unm_glm <- function(
 
     confounder1_nuisance_priors <- switch(
       family2$family,
-      "gaussian" = g("tau_{u1} ~ dt(0, 1, 3) I(0, )"),
+      "gaussian" = g("tau_{u1} ~ dgamma(0.001, 0.001)"),
       "binomial" = " ",
       "none" = ""
     )
@@ -210,7 +548,7 @@ unm_glm <- function(
 
     confounder2_nuisance_priors <- switch(
       family3$family,
-      "gaussian" = g("tau_{u2} ~ dt(0, 1, 3) I(0, )"),
+      "gaussian" = g("tau_{u2} ~ dgamma(0.001, 0.001)"),
       "binomial" = " ",
       "none" = ""
     )
@@ -250,8 +588,14 @@ unm_glm <- function(
   }
   (X <- X[, setdiff(colnames(X), colnames(U)), drop = FALSE])
 
-  (U2 <- W[, c(u2), drop = FALSE])
-  (W <- W[, setdiff(colnames(W), c(u2)), drop = FALSE])
+  if (!is.null(u2) && !is.null(W) &&
+      grepl(paste(g("\\b{u2}\\b")), deparse(form2[[3]]))) {
+    (U2 <- W[, c(u2), drop = FALSE])
+  } else {
+    U2 <- NULL
+  }
+
+  (W <- W[, setdiff(colnames(W), colnames(U2)), drop = FALSE])
 
   # determine numbers of parameters
   p_be <- ncol(X)  # = # non-confounder params in response model
@@ -271,16 +615,16 @@ unm_glm <- function(
   pretty_W_vars <- gsub("\\(Intercept\\)", "1", W_vars)
   pretty_V_vars <- gsub("\\(Intercept\\)", "1", V_vars)
 
-  # make priors
-  jags_coefs <-
-    if (is.null(u1)) {
-      c( g("beta[{1:p_be}]") )
-    } else if (is.null(u2)) {
-      c( g("beta[{1:p_be}]"), g("lambda[{1:p_la}]"), g("gamma[{1:p_ga}]") )
-    } else {
-      c( g("beta[{1:p_be}]"), g("lambda[{1:p_la}]"), g("gamma[{1:p_ga}]"),
-         g("zeta[{1:p_ze}]"), g("delta[{1:p_de}]") )
-    }
+
+  # Make priors
+  jags_coefs <- c(
+    g("beta[{1:p_be}]"),
+    if (!is.null(p_la)) g("lambda[{1:p_la}]"),
+    if (!is.null(p_ga)) g("gamma[{1:p_ga}]"),
+    if (!is.null(p_ze)) g("zeta[{1:p_ze}]"),  # Only include if p_ze > 0
+    if (!is.null(p_de)) g("delta[{1:p_de}]")  # Only include if p_de > 0
+  )
+
 
   real_coefs <- c( g("beta[{X_vars}]"), g("lambda[{U_vars}]"),
                    g("gamma[{W_vars}]"), g("zeta[{U2_vars}]"),
@@ -392,7 +736,7 @@ unm_glm <- function(
   # compile chain and adapt
   jm <- jags.model(filename, data = jd,
                    n.adapt = n.adapt, n.chains = n.chains,
-                   quiet = TRUE, ...
+                   quiet = quiet# ...
   )
 
   params_of_interest <- unique(sub("\\[.+\\]", "", real_coefs))
